package org.openxma.dsl.common;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.mwe.core.resources.ResourceLoaderFactory;

public class DefaultDslProperties implements DslProperties{
	private final static Log log = LogFactory.getLog(DefaultDslProperties.class);
	private Properties properties;

	public DefaultDslProperties(Properties properties) {
		initializeProperties(properties);
	}

	private void initializeProperties(Properties properties) {
		Properties defaultProperties = new Properties();
		loadProperties(defaultProperties, DslConstants.DEFAULT_PROPERTIES);
		Properties classPathProperties = new Properties(defaultProperties);
		loadProperties(classPathProperties, DslConstants.CLASSPATH_PROPERTIES);
		Properties systemProperties = new Properties(classPathProperties);
		String propertyFile = System.getProperty(DslConstants.CLASSPATH_PROPERTIES);
		if (propertyFile != null) {
			loadProperties(systemProperties, propertyFile);
		}
		if (properties != null) {
			mergePropertiesIntoMap(properties, systemProperties);
		}
		this.properties = systemProperties;
	}

	private void loadProperties(Properties properties, String resource) {
		InputStream resourceInputStream = ResourceLoaderFactory.createResourceLoader().getResourceAsStream(resource);
		if (resourceInputStream != null) {
			try {
				properties.load(resourceInputStream);
			} catch (IOException exception) {
				log.error(exception);
			}
		}
	}

	private void mergePropertiesIntoMap(Properties props, Map<Object, Object> map) {
		if (map == null) {
			throw new IllegalArgumentException("parameter 'map' must not be null");
		}
		if (props != null) {
			for (Enumeration<?> en = props.propertyNames(); en.hasMoreElements();) {
				String key = (String) en.nextElement();
				map.put(key, props.getProperty(key));
			}
		}
	}

	public String getProperty(String key) {
		return this.properties.getProperty(key);
	}
	
	public String getProperty(String key,String defaultValue) {
		return this.properties.getProperty(key,defaultValue);
	}
}
