package org.openxma.dsl.common;

import java.util.Properties;

/**
 * Provides access to dsl and generator properties.
 */
public interface DslProperties {

	String getProperty(String key);

	String getProperty(String key, String defaultValue);

	class Holder {
		private static final ThreadLocal<DslProperties> dslPropertiesHolder = new ThreadLocal<DslProperties>();

		/**
		 * Reset the DslProperties for the current thread.
		 */
		public static void resetDslProperties() {
			dslPropertiesHolder.set(null);
		}

		/**
		 * Associate the given DslProperties with the current thread, <i>not</i>
		 * exposing it as inheritable for child threads.
		 * 
		 * @param dslPropertyAccess
		 *            the current DslProperties, or <code>null</code> to reset
		 *            the thread-bound context
		 */
		public static void setDslProperties(DslProperties dslPropertyAccess) {
			dslPropertiesHolder.set(dslPropertyAccess);
		}

		/**
		 * Return the DslProperties associated with the current thread, if any.
		 * 
		 * @return the current DslProperties
		 */
		public static DslProperties getDslProperties() {
			DslProperties dslPropertyAccess = dslPropertiesHolder.get();
			if (dslPropertyAccess == null) {
				dslPropertyAccess = setProperties(null);
			}
			return dslPropertyAccess;
		}

		/**
		 * Associate the given generator properties with the current thread.
		 * <p>
		 * Will implicitly create a DslProperties for the given properties.
		 * 
		 * @param properties
		 *            the dsl <code>Properties</code> used by the generator, or
		 *            <code>null</code> to reset the thread-bound context
		 */
		public static DslProperties setProperties(Properties properties) {
			DslProperties dslPropertyAccess = new DefaultDslProperties(properties);
			setDslProperties(dslPropertyAccess);
			return dslPropertyAccess;
		}

		public static String getProperty(String key) {
			return getDslProperties().getProperty(key);
		}
	}
}
