package org.openxma.dsl.common.extensions;

import java.util.Map;

import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;

import com.google.common.collect.Maps;

/**
 * Simple adapter replacement for the functionality provided by
 * <code>ElementPropertiesExtensions</code> from xtend.
 * 
 */
public class EObjectPropertiesAdapter extends AdapterImpl {
	private Map<Object, Object> properties = Maps.newConcurrentHashMap();

	public EObjectPropertiesAdapter() {
	}

	public static void removeProperty(Notifier notifier, String key) {
		getAdapter(notifier).properties.remove(key);
	}

	public static void setProperty(Notifier notifier, String key, Object value) {
		getAdapter(notifier).properties.put(key, value);
	}

	public static Object getProperty(Notifier notifier, String key) {
		return getAdapter(notifier).properties.get(key);
	}

	public static boolean hasProperty(Notifier notifier, String key) {
		return getAdapter(notifier).properties.containsKey(key);
	}

	private static synchronized EObjectPropertiesAdapter getAdapter(Notifier notifier) {
		EObjectPropertiesAdapter adapter = (EObjectPropertiesAdapter) EcoreUtil.getAdapter(notifier.eAdapters(),
				EObjectPropertiesAdapter.class);
		if (adapter == null) {
			adapter = new EObjectPropertiesAdapter();
			notifier.eAdapters().add(adapter);
		}
		return adapter;
	}

	@Override
	public boolean isAdapterForType(Object type) {
		return type == getClass();
	}

}
